@@ -29,8 +29,8 @@ from cinder import context
 from cinder import exception
 from cinder.image import glance
 from cinder import service_auth
-from cinder import test
 from cinder.tests.unit.glance import stubs as glance_stubs
+from cinder.tests.unit import test
 
 
 CONF = cfg.CONF
@@ -773,7 +773,6 @@ class TestGlanceImageService(test.TestCase):
         self.flags(allowed_direct_url_schemes=['file'])
         self.service.download(self.context, image_id, writer)
         mock_copyfileobj.assert_called_once_with(mock.ANY, writer)
-        mock_open.assert_called_once_with('/tmp/test', 'rb')
 
     @mock.patch('six.moves.builtins.open')
     @mock.patch('shutil.copyfileobj')
