@@ -17,26 +17,31 @@
 import errno
 import os
 from unittest import mock
-import uuid
 
+import castellan
 import ddt
 from oslo_utils import imageutils
 from oslo_utils import units
-import six
 
 from cinder import context
 from cinder import exception
 from cinder.image import image_utils
-from cinder import test
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
+from cinder.tests.unit.keymgr import fake as fake_keymgr
+from cinder.tests.unit import test
 from cinder.volume import configuration as conf
 from cinder.volume.drivers import nfs
 from cinder.volume.drivers import remotefs
 from cinder.volume import volume_utils
 
 
+class KeyObject(object):
+    def get_encoded(arg):
+        return "asdf".encode('utf-8')
+
+
 class RemoteFsDriverTestCase(test.TestCase):
     TEST_FILE_NAME = 'test.txt'
     TEST_EXPORT = 'nas-host1:/export'
@@ -376,6 +381,57 @@ Format specific information:
     corrupt: false
     """
 
+QEMU_IMG_INFO_OUT5 = """image: volume-%(volid)s.%(snapid)s
+file format: qcow2
+virtual size: %(size_gb)sG (%(size_b)s bytes)
+disk size: 196K
+encrypted: yes
+cluster_size: 65536
+backing file: volume-%(volid)s
+backing file format: raw
+Format specific information:
+    compat: 1.1
+    lazy refcounts: false
+    refcount bits: 16
+    encrypt:
+        ivgen alg: plain64
+        hash alg: sha256
+        cipher alg: aes-256
+        uuid: 386f8626-33f0-4683-a517-78ddfe385e33
+        format: luks
+        cipher mode: xts
+        slots:
+            [0]:
+                active: true
+                iters: 1892498
+                key offset: 4096
+                stripes: 4000
+            [1]:
+                active: false
+                key offset: 262144
+            [2]:
+                active: false
+                key offset: 520192
+            [3]:
+                active: false
+                key offset: 778240
+            [4]:
+                active: false
+                key offset: 1036288
+            [5]:
+                active: false
+                key offset: 1294336
+            [6]:
+                active: false
+                key offset: 1552384
+            [7]:
+                active: false
+                key offset: 1810432
+        payload offset: 2068480
+        master key iters: 459347
+    corrupt: false
+"""
+
 
 @ddt.ddt
 class NfsDriverTestCase(test.TestCase):
@@ -395,7 +451,6 @@ class NfsDriverTestCase(test.TestCase):
     TEST_SHARES_CONFIG_FILE = '/etc/cinder/test-shares.conf'
     TEST_NFS_EXPORT_SPACES = 'nfs-host3:/export this'
     TEST_MNT_POINT_SPACES = '/ 0 0 0 /foo'
-    VOLUME_UUID = '69ad4ff6-b892-4215-aaaa-aaaaaaaaaaaa'
 
     def setUp(self):
         super(NfsDriverTestCase, self).setUp()
@@ -694,6 +749,17 @@ class NfsDriverTestCase(test.TestCase):
                                            provider_location=loc,
                                            size=size)
 
+    def _simple_encrypted_volume(self, size=10):
+        loc = self.TEST_NFS_EXPORT1
+        info_dic = {'name': u'volume-0000000a',
+                    'id': '55555555-222f-4b32-b585-9991b3bf0a99',
+                    'size': size,
+                    'encryption_key_id': fake.ENCRYPTION_KEY_ID}
+
+        return fake_volume.fake_volume_obj(self.context,
+                                           provider_location=loc,
+                                           **info_dic)
+
     def test_get_provisioned_capacity(self):
         self._set_driver()
         drv = self._driver
@@ -997,6 +1063,28 @@ class NfsDriverTestCase(test.TestCase):
                         resize.assert_called_once_with(path, newSize,
                                                        run_as_root=True)
 
+    def test_extend_volume_attached_fail(self):
+        """Extend a volume by 1."""
+        self._set_driver()
+        drv = self._driver
+        volume = fake_volume.fake_volume_obj(
+            self.context,
+            id='80ee16b6-75d2-4d54-9539-ffc1b4b0fb10',
+            size=1,
+            provider_location='nfs_share')
+        path = 'path'
+        newSize = volume['size'] + 1
+
+        with mock.patch.object(drv, 'local_path', return_value=path):
+            with mock.patch.object(drv, '_is_share_eligible',
+                                   return_value=True):
+                with mock.patch.object(drv, '_is_file_size_equal',
+                                       return_value=True):
+                    with mock.patch.object(drv, '_is_volume_attached',
+                                           return_value=True):
+                        self.assertRaises(exception.ExtendVolumeError,
+                                          drv.extend_volume, volume, newSize)
+
     def test_extend_volume_failure(self):
         """Error during extend operation."""
         self._set_driver()
@@ -1127,14 +1215,15 @@ class NfsDriverTestCase(test.TestCase):
         """Case where the mount works the first time."""
 
         self._set_driver()
-        self.mock_object(self._driver._remotefsclient, 'mount')
+        self.mock_object(self._driver._remotefsclient, 'mount', autospec=True)
         drv = self._driver
         drv.configuration.nfs_mount_attempts = 3
         drv.shares = {self.TEST_NFS_EXPORT1: ''}
 
         drv._ensure_share_mounted(self.TEST_NFS_EXPORT1)
 
-        drv._remotefsclient.mount.called_once()
+        drv._remotefsclient.mount.assert_called_once_with(
+            self.TEST_NFS_EXPORT1, [])
 
     @mock.patch('time.sleep')
     def test_ensure_share_mounted_exception(self, _mock_sleep):
@@ -1172,16 +1261,48 @@ class NfsDriverTestCase(test.TestCase):
         self.assertEqual(min_num_attempts,
                          drv._remotefsclient.mount.call_count)
 
-    @ddt.data([NFS_CONFIG1, QEMU_IMG_INFO_OUT3],
-              [NFS_CONFIG2, QEMU_IMG_INFO_OUT4],
-              [NFS_CONFIG3, QEMU_IMG_INFO_OUT3],
-              [NFS_CONFIG4, QEMU_IMG_INFO_OUT4])
+    @mock.patch('tempfile.NamedTemporaryFile')
+    @ddt.data([NFS_CONFIG1, QEMU_IMG_INFO_OUT3, False],
+              [NFS_CONFIG2, QEMU_IMG_INFO_OUT4, False],
+              [NFS_CONFIG3, QEMU_IMG_INFO_OUT3, False],
+              [NFS_CONFIG4, QEMU_IMG_INFO_OUT4, False],
+              [NFS_CONFIG4, QEMU_IMG_INFO_OUT5, True])
     @ddt.unpack
-    def test_copy_volume_from_snapshot(self, nfs_conf, qemu_img_info):
+    def test_copy_volume_from_snapshot(self, nfs_conf, qemu_img_info,
+                                       encryption, mock_temp_file):
+
+        class DictObj(object):
+            # convert a dict to object w/ attributes
+            def __init__(self, d):
+                self.__dict__ = d
+
         self._set_driver(extra_confs=nfs_conf)
         drv = self._driver
-        dest_volume = self._simple_volume()
-        src_volume = self._simple_volume()
+
+        src_encryption_key_id = None
+        dest_encryption_key_id = None
+
+        if encryption:
+            mock_temp_file.return_value.__enter__.side_effect = [
+                DictObj({'name': '/tmp/imgfile'}),
+                DictObj({'name': '/tmp/passfile'})]
+
+            dest_volume = self._simple_encrypted_volume()
+            src_volume = self._simple_encrypted_volume()
+
+            key_mgr = fake_keymgr.fake_api()
+            self.mock_object(castellan.key_manager, 'API',
+                             return_value=key_mgr)
+            key_id = key_mgr.store(self.context, KeyObject())
+
+            src_volume.encryption_key_id = key_id
+            dest_volume.encryption_key_id = key_id
+
+            src_encryption_key_id = src_volume.encryption_key_id
+            dest_encryption_key_id = dest_volume.encryption_key_id
+        else:
+            dest_volume = self._simple_volume()
+            src_volume = self._simple_volume()
 
         fake_snap = fake_snapshot.fake_snapshot_obj(self.context)
         fake_snap.volume = src_volume
@@ -1212,16 +1333,25 @@ class NfsDriverTestCase(test.TestCase):
 
         mock_permission = self.mock_object(drv, '_set_rw_permissions_for_all')
 
-        drv._copy_volume_from_snapshot(fake_snap, dest_volume, size)
+        drv._copy_volume_from_snapshot(fake_snap, dest_volume, size,
+                                       src_encryption_key_id,
+                                       dest_encryption_key_id)
 
         mock_read_info_file.assert_called_once_with(info_path)
         mock_img_info.assert_called_once_with(snap_path,
                                               force_share=True,
                                               run_as_root=True)
         used_qcow = nfs_conf['nfs_qcow2_volumes']
-        mock_convert_image.assert_called_once_with(
-            src_vol_path, dest_vol_path, 'qcow2' if used_qcow else 'raw',
-            run_as_root=True)
+        if encryption:
+            mock_convert_image.assert_called_once_with(
+                src_vol_path, dest_vol_path, 'luks',
+                passphrase_file='/tmp/passfile',
+                run_as_root=True,
+                src_passphrase_file='/tmp/imgfile')
+        else:
+            mock_convert_image.assert_called_once_with(
+                src_vol_path, dest_vol_path, 'qcow2' if used_qcow else 'raw',
+                run_as_root=True)
         mock_permission.assert_called_once_with(dest_vol_path)
 
     @ddt.data([NFS_CONFIG1, QEMU_IMG_INFO_OUT3, 'available'],
@@ -1237,7 +1367,7 @@ class NfsDriverTestCase(test.TestCase):
         # Volume source of the snapshot we are trying to clone from. We need it
         # to have a different id than the default provided.
         src_volume = self._simple_volume(size=10)
-        src_volume.id = six.text_type(uuid.uuid4())
+        src_volume.id = fake.VOLUME_ID
         src_volume_dir = os.path.join(self.TEST_MNT_POINT_BASE,
                                       drv._get_hash_str(
                                           src_volume.provider_location))
@@ -1378,8 +1508,6 @@ class NfsDriverTestCase(test.TestCase):
                 mock.patch.object(drv, '_read_info_file', return_value={}), \
                 mock.patch.object(drv, '_do_create_snapshot') \
                 as mock_do_create_snapshot, \
-                mock.patch.object(drv, '_check_snapshot_support') \
-                as mock_check_support, \
                 mock.patch.object(drv, '_write_info_file') \
                 as mock_write_info_file, \
                 mock.patch.object(drv, 'get_active_image_from_info',
@@ -1388,7 +1516,6 @@ class NfsDriverTestCase(test.TestCase):
                                   return_value=snap_path):
             self._driver.create_snapshot(fake_snap)
 
-        mock_check_support.assert_called_once()
         mock_do_create_snapshot.assert_called_with(fake_snap, volume['name'],
                                                    snap_path)
         mock_write_info_file.assert_called_with(
@@ -1462,6 +1589,7 @@ class NfsDriverDoSetupTestCase(test.TestCase):
         super(NfsDriverDoSetupTestCase, self).setUp()
         self.context = mock.Mock()
         self.create_configuration()
+        self.override_config('compute_api_class', 'unittest.mock.Mock')
 
     def create_configuration(self):
         config = conf.Configuration(None)
