@@ -16,12 +16,13 @@
 Tests for Zadara VPSA volume driver
 """
 import copy
-import mock
+from unittest import mock
+
 import requests
 from six.moves.urllib import parse
 
 from cinder import exception
-from cinder import test
+from cinder.tests.unit import test
 from cinder.volume import configuration as conf
 from cinder.volume.drivers import zadara
 
