@@ -29,10 +29,10 @@ from cinder import exception
 from cinder import objects
 from cinder.objects import fields
 from cinder import quota
-from cinder import test
 from cinder.tests.unit.api.v2 import fakes as v2_fakes
 from cinder.tests.unit.brick import fake_lvm
 from cinder.tests.unit import fake_constants as fake
+from cinder.tests.unit import test
 from cinder.tests.unit import utils as tests_utils
 from cinder.tests.unit import volume as base
 import cinder.volume
