@@ -76,7 +76,9 @@ RBD_OPTS = [
                default=5,
                help='Maximum number of nested volume clones that are '
                     'taken before a flatten occurs. Set to 0 to disable '
-                    'cloning.'),
+                    'cloning. Note: lowering this value will not affect '
+                    'existing volumes whose clone depth exceeds the new '
+                    'value.'),
     cfg.IntOpt('rbd_store_chunk_size', default=4,
                help='Volumes will be chunked into objects of this size '
                     '(in megabytes).'),
@@ -246,6 +248,7 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
         self._is_replication_enabled = False
         self._replication_targets = []
         self._target_names = []
+        self._clone_v2_api_checked = False
 
         if self.rbd is not None:
             self.RBD_FEATURE_LAYERING = self.rbd.RBD_FEATURE_LAYERING
@@ -292,6 +295,22 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
             'max_over_subscription_ratio', 'volume_dd_blocksize')
         return RBD_OPTS + additional_opts
 
+    def _show_msg_check_clone_v2_api(self, volume_name):
+        if not self._clone_v2_api_checked:
+            self._clone_v2_api_checked = True
+            with RBDVolumeProxy(self, volume_name) as volume:
+                try:
+                    if (volume.volume.op_features() &
+                            self.rbd.RBD_OPERATION_FEATURE_CLONE_PARENT):
+                        LOG.info('Using v2 Clone API')
+                        return
+                except AttributeError:
+                    pass
+                LOG.warning('Not using v2 clone API, please upgrade to'
+                            ' mimic+ and set the OSD minimum client'
+                            ' compat version to mimic for better'
+                            ' performance, fewer deletion issues')
+
     def _get_target_config(self, target_id):
         """Get a replication target from known replication targets."""
         for target in self._replication_targets:
@@ -556,14 +575,14 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
 
         with RADOSClient(self) as client:
             ret, df_outbuf, __ = client.cluster.mon_command(
-                '{"prefix":"df", "format":"json"}', '')
+                '{"prefix":"df", "format":"json"}', b'')
             if ret:
                 LOG.warning('Unable to get rados pool stats.')
                 return 'unknown', 'unknown'
 
             ret, quota_outbuf, __ = client.cluster.mon_command(
                 '{"prefix":"osd pool get-quota", "pool": "%s",'
-                ' "format":"json"}' % pool_name, '')
+                ' "format":"json"}' % pool_name, b'')
             if ret:
                 LOG.warning('Unable to get rados pool quotas.')
                 return 'unknown', 'unknown'
@@ -656,12 +675,6 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
         if not parent:
             return depth
 
-        # If clone depth was reached, flatten should have occurred so if it has
-        # been exceeded then something has gone wrong.
-        if depth > self.configuration.rbd_max_clone_depth:
-            raise Exception(_("clone depth exceeds limit of %s") %
-                            (self.configuration.rbd_max_clone_depth))
-
         return self._get_clone_depth(client, parent, depth + 1)
 
     def _extend_if_required(self, volume, src_vref):
@@ -731,16 +744,18 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
             depth = self._get_clone_depth(client, src_name)
             # If dest volume is a clone and rbd_max_clone_depth reached,
             # flatten the dest after cloning. Zero rbd_max_clone_depth means
-            # infinite is allowed.
+            # volumes are always flattened.
             if depth >= self.configuration.rbd_max_clone_depth:
                 LOG.info("maximum clone depth (%d) has been reached - "
                          "flattening dest volume",
                          self.configuration.rbd_max_clone_depth)
-                dest_volume = self.rbd.Image(client.ioctx, dest_name)
+
+                # Flatten destination volume
                 try:
-                    # Flatten destination volume
-                    LOG.debug("flattening dest volume %s", dest_name)
-                    dest_volume.flatten()
+                    with RBDVolumeProxy(self, dest_name, client=client,
+                                        ioctx=client.ioctx) as dest_volume:
+                        LOG.debug("flattening dest volume %s", dest_name)
+                        dest_volume.flatten()
                 except Exception as e:
                     msg = (_("Failed to flatten volume %(volume)s with "
                              "error: %(error)s.") %
@@ -749,8 +764,6 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
                     LOG.exception(msg)
                     src_volume.close()
                     raise exception.VolumeBackendAPIException(data=msg)
-                finally:
-                    dest_volume.close()
 
                 try:
                     # remove temporary snap
@@ -1032,6 +1045,8 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
             self._flatten(self.configuration.rbd_pool, volume.name)
         if int(volume.size):
             self._resize(volume)
+
+        self._show_msg_check_clone_v2_api(snapshot.volume_name)
         return volume_update
 
     def _delete_backup_snaps(self, rbd_image):
@@ -1608,7 +1623,13 @@ class RBDDriver(driver.CloneableImageVD, driver.MigrateVD,
             if encrypted:
                 self._encrypt_image(context, volume, tmp_dir, tmp.name)
 
-            self.delete_volume(volume)
+            @utils.retry(exception.VolumeIsBusy,
+                         self.configuration.rados_connection_interval,
+                         self.configuration.rados_connection_retries)
+            def _delete_volume(volume):
+                self.delete_volume(volume)
+
+            _delete_volume(volume)
 
             chunk_size = self.configuration.rbd_store_chunk_size * units.Mi
             order = int(math.log(chunk_size, 2))
